<?php
// $Id: dbi_mysqli.php,v 1.8 2002/02/11 15:09:27 erwin Exp $

// This class is based on the Perl DBI.
// New functionality should be added accordingly.
// Please refer to the DBI documentation (perldoc DBI).

/**
 * Class DBI
 */
class DBI extends STH
{

  /**
   * @var
   */
  public $sName;

  /**
   * @var mysqli
   */
  public mysqli $oSession;
  /**
   * @var bool
   */
  public bool $bDebug = false;

  /**
   * @var
   */
  public $err;
  /**
   * @var string
   */
  public $errstr = '';
  /**
   * @var bool
   */
  public $bRaiseError = false;

  /**
   * DBI constructor.
   * @param string $db
   * @param string $user
   * @param string $password
   * @param string $host
   * @param string $charset
   */
  function __construct($db, $user, $password, string $host, string $charset = 'utf8')
  {
    $this->oSession = @mysqli_connect($host, $user, $password, $db);
    if (@mysqli_connect_errno()) {
      echo 'Vehler bei der Verbindung zu MySQL: ' . mysqli_connect_error();
    }
    if (is_object($this->oSession) && !is_null($charset)) {
      if (!mysqli_set_charset($this->oSession, $charset)) {
        printf('Fehler beim Laden des Zeichensatz %s: %s' . "\n", $charset, mysqli_error($this->oSession));
        exit();
      }
    }
    $this->sName = $db;
  }



  /**
   * @param $query
   * @return $this
   */
  function prepare($query): static {
    parent::__construct($this->oSession, $query, $this->bDebug);
    return $this;
  }

  /**
   * @param $query
   * @return int
   */
  function dbh_do($query)
  {
    $res = mysqli_query($query, $this->oSession);
    if ($res) {
      $this->err = 0;
      $this->errstr = '';
      return mysqli_affected_rows($this->oSession);
    } else {
      $this->err = mysqli_errno($this->oSession);
      $this->errstr = mysqli_error($this->oSession);
      return -1;
    }
  }



  /**
   * @param $query
   * @return string
   */
  function escape_string($query)
  {
    return mysqli_escape_string($this->oSession, $query);
  }

}

/**
 * Class STH
 */
class STH
{
  /**
   * @var
   */
  public $query;
  /**
   * @var bool
   */
  public bool $bDebug;
  /**
   * @var mysqli
   */
  public mysqli $oSession;
  /**
   * @var
   */
  public $res;
  /**
   * @var
   */
  public $row;
  /**
   * @var array
   */
  public $placeholders;
  /**
   * @var
   */
  public $dbi;

  /**
   * STH constructor.
   * @param mysqli $oSession
   * @param string $query
   * @param $bDebug
   */
  function __construct(mysqli $oSession, string $query, $bDebug)
  {
    $this->oSession = $oSession;
    $this->query = $query;
    $this->bDebug = $bDebug;


    $this->placeholders = array();
    $quote = '';
    for ($i = 0; $i < strlen($query); ++$i) {
      if ($query[$i] == '\'') {
        if (empty($quote)) {
          $quote = '\'';
        } elseif ($quote == '\'') {
          $quote = '';
        }
      } elseif ($query[$i] == '"') {
        if (empty($quote)) {
          $quote = '"';
        } elseif ($quote == '"') {
          $quote = '';
        }
      } elseif ($query[$i] == '?') {
        if (empty($quote)) {
          $this->placeholders[] = $i;
        }
      }
    }
  }

  /**
   * @return bool|mysqli_result
   */
  function execute(): mysqli_result|bool {

    $numargs = func_num_args();
    $arg_list = func_get_args();

    $parms = array();

    for ($i = 0; $i < $numargs; $i++) {
      if (is_array($arg_list[$i])) {
        foreach ($arg_list[$i] as $parm) {
          $parms[] = $parm;
        }
      } else {
        $parms[] = $arg_list[$i];
      }
    }

    if (sizeof($parms) != sizeof($this->placeholders)) {
      print '<br><b>SQL Query contains ' . sizeof($this->placeholders) . ' placeholders but ' . sizeof($parms) . ' was passed</b><br>';
      exit;
    }

    if (sizeof($parms) > 0) {
      $query = substr($this->query, 0, $this->placeholders[0]);
      for ($i = 0; $i < sizeof($parms) - 1; ++$i) {
        $query .= $this->quote($parms[$i]) . substr($this->query, $this->placeholders[$i] + 1, $this->placeholders[$i + 1] - $this->placeholders[$i] - 1);
      }
      $query .= $this->quote($parms[$i]) . substr($this->query, $this->placeholders[$i] + 1);
    } else {
      $query = $this->query;
    }

    if ($this->bDebug) {
      $fd = @fopen('./tmp/dbi.' . $_SERVER['SERVER_NAME'] . '.log', 'a') or die ('Couldn\'t append to file');
      fputs($fd, $query . "\n==================\n");
      fclose($fd);
    }

    $this->res = @mysqli_query($this->oSession, $query);
    if (!$this->res) {
      $this->dbi=new stdClass();
      if (isset($this->dbi->bRaiseError) && $this->dbi->bRaiseError) {
        print '<br><b>Could not execute SQL query: " ' . $query . '"</b><br>';
        print '<br>' . mysqli_errno($this->oSession) . ' ' . mysqli_error($this->oSession) . '<br>';
        exit;
      }
      $this->dbi->errstr = mysqli_error($this->oSession);
      $this->dbi->err = mysqli_errno($this->oSession);
    }
    $this->row = 0;
    return $this->res;
  }

  /**
   * @return array|false|null
   */
  function fetchrow_array(): false|array|null {
    $this->row++;
    return mysqli_fetch_row($this->res);
  }

  /**
   * @return array|false|null
   */
  function fetchrow_hash(): false|array|null {
    $this->row++;
    return mysqli_fetch_array($this->res, MYSQLI_ASSOC);
  }

  /**
   *
   */
  function finish(): void {
    if(isset($this->res)) {
      mysqli_free_result($this->res);
    }
  }

  /**
   * @return int|string
   */
  function insert_id(): int|string {
    return mysqli_insert_id($this->oSession);
  }

  /**
   * @return int|string
   */
  function rows(): int|string {
    return mysqli_num_rows($this->res);
  }

  /**
   * @return int
   */
  function num_tablefields(): int {
    return mysqli_num_fields($this->res);
  }

  /**
   * @param $str
   * @return string
   */
  function quote($str): string {
    if (function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc()) {
      $str = stripslashes($str);
    }
    return '\'' . addslashes($str) . '\'';
  }
  function __destruct() {
    $this->finish();
  }
}